package tareas.tarea2.gui;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import tareas.tarea2.electronica.FuenteCC;
import tareas.tarea2.electronica.GND;
import tareas.tarea2.electronica.Resistencia;

/**
 * Clase que implementa la Barra de menu.
 * @author Janther
 */
public class Barra extends JMenuBar {

    /**
     * Numero Serialde la clase.
     */
    static final long serialVersionUID = 1;
    /**
     * Constructor de la Barra.
     * @param dib Es el panel donde se mostraran los componentes electricos.
     */
    public Barra(final Dibujo dib) {
        JMenuItem menuItem;
        JMenu menuCrear = new JMenu("Crear");
        menuCrear.setMnemonic(KeyEvent.VK_C);
        JMenu menuModificador = new JMenu("Modificador");
        menuModificador.setMnemonic(KeyEvent.VK_M);

        menuItem = new JMenuItem("Fuente de Voltaje");
        menuItem.setMnemonic(KeyEvent.VK_V);
        menuItem.addActionListener(new ActionListener() {
            public void actionPerformed(final ActionEvent e) {
                DefaultListener defaultListener = new DefaultListener(dib);
                dib.changeMouseListener(defaultListener);
                dib.changeMouseMotionListener(defaultListener);
                Object[] message = new Object[2];
                //Primer objeto es un Label.
                message[0] = "Valor de la fuente de Voltaje:";
                //El segundo objeto es un Panel con las opciones.
                JPanel panel = new JPanel();
                panel.setLayout(new BoxLayout(panel, BoxLayout.X_AXIS));
                JSpinner v =
                    new JSpinner(new SpinnerNumberModel(0.0, null, null, 1));
                panel.add(v);
                panel.add(new JLabel("[V]"));
                message[1] = panel;
                String[] options = {
                        "Aceptar",
                        "Cancelar"
                };
                //Se instancia la ventana.
                int result = JOptionPane.showOptionDialog(
                        null,
                        message,
                        "Editor de Voltaje",
                        JOptionPane.DEFAULT_OPTION,
                        JOptionPane.INFORMATION_MESSAGE,
                        new ImageIcon(getClass().getResource(
                                "/iconos/FuenteCC.jpg")),
                        options,
                        options[0]
                );
                switch(result) {
                case 0:
                    //Opcion Aceptar
                    dib.addComponente(new FuenteCC((Double) v.getValue()));
                    break;
                case 1:
                    //Opcion Cancelar
                    break;
                default:
                    break;
                }
            }
        });
        menuCrear.add(menuItem);

        menuItem = new JMenuItem("Resistencia");
        menuItem.setMnemonic(KeyEvent.VK_R);
        menuItem.addActionListener(new ActionListener() {
            public void actionPerformed(final ActionEvent e) {
                DefaultListener defaultListener = new DefaultListener(dib);
                dib.changeMouseListener(defaultListener);
                dib.changeMouseMotionListener(defaultListener);
                Object[] message = new Object[2];
                //Primer objeto es un Label.
                message[0] = "Valor de la Resistencia:";
                //El segundo objeto es un Panel con las opciones.
                JPanel panel = new JPanel();
                panel.setLayout(new BoxLayout(panel, BoxLayout.X_AXIS));
                JSpinner r =
                    new JSpinner(new SpinnerNumberModel(0.0, null, null, 1));
                panel.add(r);
                panel.add(new JLabel("[ohm]"));
                message[1] = panel;
                String[] options = {
                        "Aceptar",
                        "Cancelar"
                };
                //Se instancia la ventana.
                int result = JOptionPane.showOptionDialog(
                        null,
                        message,
                        "Editor de Resistencia",
                        JOptionPane.DEFAULT_OPTION,
                        JOptionPane.INFORMATION_MESSAGE,
                        new ImageIcon(getClass().getResource(
                                "/iconos/Resistencia.jpg")),
                        options,
                        options[0]
                );
                switch(result) {
                case 0:
                    //Opcion Aceptar
                    dib.addComponente(new Resistencia((Double) r.getValue()));
                    break;
                case 1:
                    //Opcion Cancelar
                    break;
                default:
                    break;
                }
            }
        });
        menuCrear.add(menuItem);

        menuItem = new JMenuItem("Tierra");
        menuItem.setMnemonic(KeyEvent.VK_T);
        menuItem.addActionListener(new ActionListener() {
            public void actionPerformed(final ActionEvent e) {
                DefaultListener defaultListener = new DefaultListener(dib);
                dib.changeMouseListener(defaultListener);
                dib.changeMouseMotionListener(defaultListener);
                dib.addComponente(new GND());
            }
        });
        menuCrear.add(menuItem);

        menuItem = new JMenuItem("Cable");
        menuItem.setMnemonic(KeyEvent.VK_T);
        menuItem.addActionListener(new UnirListener(dib));
        menuCrear.add(menuItem);

        menuItem = new JMenuItem("Default");
        menuItem.setMnemonic(KeyEvent.VK_D);
        menuItem.addActionListener(new DefaultListener(dib));
        menuModificador.add(menuItem);

        menuItem = new JMenuItem("Trasladar");
        menuItem.setMnemonic(KeyEvent.VK_A);
        menuItem.addActionListener(new TrasladarListener(dib));
        menuModificador.add(menuItem);

        menuItem = new JMenuItem("Rotar");
        menuItem.setMnemonic(KeyEvent.VK_O);
        menuItem.addActionListener(new RotarListener(dib));
        menuModificador.add(menuItem);

        menuItem = new JMenuItem("Editar");
        menuItem.setMnemonic(KeyEvent.VK_I);
        menuItem.addActionListener(new EditarListener(dib));
        menuModificador.add(menuItem);

        menuItem = new JMenuItem("Eliminar");
        menuItem.setMnemonic(KeyEvent.VK_L);
        menuItem.addActionListener(new DestroyListener(dib));
        menuModificador.add(menuItem);

        add(menuCrear);
        add(menuModificador);
    }
}
