package tareas.tarea2.gui;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import javax.swing.SwingUtilities;

/**
 * Clase que implementa los metodos de ambos Listener pedidos en la tarea, mas
 * uno que edita los valores de la resistencia y fuente para tener un control
 * sobre el dibujo mas natural.
 * @author Klaus
 */
public class DefaultListener implements ActionListener, MouseListener,
        MouseMotionListener {
    /**
     * Panel donde se efectuaran los cambios mediante el mouse.
     */
    private Dibujo dibujo;
    /**
     * Listener que rota los componentes en noventa grados.
     */
    private RotarListener r;
    /**
     * Listener que traslada los componentes.
     */
    private TrasladarListener t;
    /**
     * Listener que edita los valores de las componentes.
     */
    private EditarListener e;

    /**
     * Constructor de la clase.
     * @param dib Panel donde se efectuaran los cambios mediante el mouse.
     */
    public DefaultListener(Dibujo dib) {
        dibujo = dib;
        r = new RotarListener(dib);
        t = new TrasladarListener(dib);
        e = new EditarListener(dib);
    }

    /**
     * Metodo que se ejectuta cuando se dispara el evento ActionEvent.
     * @param event Evento de accion que pertenece al objeto al cual se le
     * asigno este Listener.
     */
    public void actionPerformed(ActionEvent event) {
        dibujo.changeMouseListener(this);
        dibujo.changeMouseMotionListener(this);
        dibujo.repaint();
    }

    /**
     * Metodo que se ejectuta cuando se presiona el mouse.
     * @param event Evento de mouse que tiene encapsulado las variables del
     * mouse.
     */
    public void mousePressed(MouseEvent event) {
        if (SwingUtilities.isLeftMouseButton(event)) {
            t.mousePressed(event);
        }
    }

    /**
     * Metodo que se ejectuta cuando se suelta el mouse.
     * @param event Evento de mouse que tiene encapsulado las variables del
     * mouse.
     */
    public void mouseReleased(MouseEvent event) {
        if (SwingUtilities.isLeftMouseButton(event)) {
            t.mouseReleased(event);
        }
    }

    /**
     * Metodo que se ejectuta cuando se hace click.
     * @param event Evento de mouse que tiene encapsulado las variables del
     * mouse.
     */
    public void mouseClicked(MouseEvent event) {
        if (SwingUtilities.isLeftMouseButton(event)) {
            if (event.getClickCount() == 2) {
                e.mouseClicked(event);
            }
        }
        if (SwingUtilities.isRightMouseButton(event)) {
            r.mouseClicked(event);
        }
    }

    /**
     * Metodo que se ejectuta cuando el mouse entra en el dibujo.
     * @param event Evento de mouse que tiene encapsulado las variables del
     * mouse.
     */
    public void mouseEntered(MouseEvent event) { }

    /**
     * Metodo que se ejectuta cuando el mouse sale del dibujo.
     * @param event Evento de mouse que tiene encapsulado las variables del
     * mouse.
     */
    public void mouseExited(MouseEvent event) { }

    /**
     * Metodo que se ejectuta cuando el mouse se arrastra una vez presionado.
     * @param event Evento de mouse que tiene encapsulado las variables del
     * mouse.
     */
    public void mouseDragged(MouseEvent event) {
        if (SwingUtilities.isLeftMouseButton(event)) {
            t.mouseDragged(event);
        }
    }

    /**
     * Metodo que se ejectuta cuando el mouse se arrastra una vez presionado.
     * @param event Evento de mouse que tiene encapsulado las variables del
     * mouse.
     */
    public void mouseMoved(MouseEvent event) {
        r.mouseMoved(event);
        e.mouseMoved(event);
        t.mouseMoved(event);
    }
}
