package tareas.tarea2.gui;

import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.util.ArrayList;
import tareas.tarea2.electronica.Cable;
import tareas.tarea2.electronica.Nodo;

/**
 * Clase que implementa los metodos, que trasladan un componente, pedidos en la
 * tarea.
 * @author Klaus
 */
public class UnirListener implements ActionListener, MouseListener,
        MouseMotionListener {

    /**
     * Panel donde se efectuaran los cambios mediante el mouse.
     */
    private Dibujo dibujo;
    /**
     * ID de la forma seleccionada.
     */
    private int selected;
    /**
     * Punto anterior usado para calcual el deltaX y el deltaY.
     */
    private Point previo;
    /**
     * Arreglo de nodos.
     */
    private ArrayList < Nodo > nodos;
    /**
     * Nodo temporal.
     */
    private Nodo nodo;
    /**
     * Cable temporal.
     */
    private Cable cable;
    /**
     * Nodo que se fijara en caso de tener una posible union.
     */
    private Nodo nodoFijo1;
    /**
     * Nodo clonado que se fijara en caso de tener una posible union.
     */
    private Nodo nodoFijo2;
    /**
     * Nodo que se fijara en caso de tener una posible union.
     */
    private Nodo nodoFijo3;
    /**
     * Estado del proceso.
     */
    private int estado;
    /**
     * Valor del estado al inicio.
     */
    private static final int INICIO = 0;
    /**
     * Valor del estado una vez que se ha seleccionado la figura.
     */
    private static final int CREATED = 1;
    /**
     * Valor del estado una vez que se ha encontrado una posible union.
     */
    private static final int UNITED = 2;

    /**
     * Constructor de la clase.
     * @param dib Panel donde se efectuaran los cambios mediante el mouse.
     */
    public UnirListener(Dibujo dib) {
        dibujo = dib;
        estado = INICIO;
        nodo = null;
        nodoFijo1 = null;
        nodoFijo2 = null;
        nodoFijo3 = null;
        selected = -1;
    }

    /**
     * Metodo que se ejectuta cuando se dispara el evento ActionEvent.
     * @param event Evento de accion que pertenece al objeto al cual se le
     * asigno este Listener.
     */
    public void actionPerformed(ActionEvent event) {
        dibujo.changeMouseListener(this);
        dibujo.changeMouseMotionListener(this);
        nodos = dibujo.getNodos(-1);
        dibujo.repaint();
    }

    /**
     * Metodo que se ejectuta cuando se presiona el mouse.
     * @param event Evento de mouse que tiene encapsulado las variables del
     * mouse.
     */
    public void mousePressed(MouseEvent event) {
        if (selected >= 0) {
            estado = CREATED;
            nodoFijo1 = nodos.get(selected);
            nodo = (Nodo) nodoFijo1.clone();
            cable = new Cable(nodoFijo1, nodo);
            dibujo.addComponente(cable);
            dibujo.addComponente(nodo);
            previo = event.getPoint();
        }
    }

    /**
     * Metodo que se ejectuta cuando se suelta el mouse.
     * @param event Evento de mouse que tiene encapsulado las variables del
     * mouse.
     */
    public void mouseReleased(MouseEvent event) {
        if (estado != INICIO) {
            dibujo.removeComponente(cable);
            cable.destroy();
            dibujo.removeComponente(nodo);
            if (estado == UNITED) {
                dibujo.addComponente(new Cable(nodoFijo1, nodoFijo2));
            }
            estado = INICIO;
            nodo = null;
            nodoFijo1 = null;
            nodoFijo2 = null;
            nodoFijo3 = null;
            selected = -1;
            dibujo.repaint();
        }
    }

    /**
     * Metodo que se ejectuta cuando se hace click.
     * @param event Evento de mouse que tiene encapsulado las variables del
     * mouse.
     */
    public void mouseClicked(MouseEvent event) { }

    /**
     * Metodo que se ejectuta cuando el mouse entra en el dibujo.
     * @param event Evento de mouse que tiene encapsulado las variables del
     * mouse.
     */
    public void mouseEntered(MouseEvent event) { }

    /**
     * Metodo que se ejectuta cuando el mouse sale del dibujo.
     * @param event Evento de mouse que tiene encapsulado las variables del
     * mouse.
     */
    public void mouseExited(MouseEvent event) { }

    /**
     * Metodo que se ejectuta cuando el mouse se arrastra una vez presionado.
     * @param event Evento de mouse que tiene encapsulado las variables del
     * mouse.
     */
    public void mouseDragged(MouseEvent event) {
        if (estado == CREATED) {
            Point p = event.getPoint();
            double deltaX = p.getX() - previo.getX();
            double deltaY = p.getY() - previo.getY();
            nodo.trasladar(deltaX, deltaY);
            dibujo.repaint();
            previo = p;
            Nodo[] n1 = nodo.getNodos();
            for (int i = 0; i < nodos.size(); i++) {
                if (nodos.get(i).intersects(n1[0])) {
                    deltaX = nodos.get(i).getPunto().getX()
                        - n1[0].getPunto().getX();
                    deltaY = nodos.get(i).getPunto().getY()
                        - n1[0].getPunto().getY();
                    nodoFijo2 = nodos.get(i);
                    nodoFijo3 = (Nodo) nodo.clone();
                    nodo.trasladar(deltaX, deltaY);
                    estado = UNITED;
                }
            }
        }
        if (estado == UNITED) {
            Point p = event.getPoint();
            double deltaX = p.getX() - previo.getX();
            double deltaY = p.getY() - previo.getY();
            nodoFijo3.trasladar(deltaX, deltaY);
            previo = p;
            if (!nodoFijo2.intersects(nodoFijo3)) {
                deltaX = nodoFijo3.getPunto().getX()
                    - nodo.getPunto().getX();
                deltaY = nodoFijo3.getPunto().getY()
                    - nodo.getPunto().getY();
                nodo.trasladar(deltaX, deltaY);
                estado = CREATED;
            }
        }
    }

    /**
     * Metodo que se ejectuta cuando el mouse se arrastra una vez presionado.
     * @param event Evento de mouse que tiene encapsulado las variables del
     * mouse.
     */
    public void mouseMoved(MouseEvent event) {
        for (int i = 0; i < nodos.size(); i++) {
            if (nodos.get(i).isSelected(event.getPoint())) {
                selected = i;
                dibujo.repaint();
                return;
            } else {
                selected = -1;
                dibujo.repaint();
            }
        }
    }
}
